# -*-coding:utf-8 -*-
u"""
:创建时间: 2022/5/29 7:15
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:QQ: 2921251087
:爱发电: https://afdian.net/@Phantom_of_the_Cang
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127

"""
from __future__ import unicode_literals, print_function, division

import sys

import cpmel.cmds as cc

from rig_operation_graph._utils import *

operation_map = {
    '==': 0,
    '!=': 1,
    '>': 2,
    '>=': 3,
    '<': 4,
    '<=': 5,
}


def float_logic(ctx, l_attr, r_attr, operation, true_attr, false_attr):
    node = cc.createNode('condition')
    set_or_connect(l_attr, node.firstTerm)
    set_or_connect(r_attr, node.secondTerm)

    node['operation'] = operation_map[operation]

    set_or_connect(true_attr, node.colorIfTrueR)
    set_or_connect(false_attr, node.colorIfFalseR)
    return node.outColorR


def float_equal(ctx, l_attr, r_attr, true_attr, false_attr):
    """等于"""
    return float_logic(ctx, l_attr, r_attr, '==', true_attr, false_attr)


def float_not_equal(ctx, l_attr, r_attr, true_attr, false_attr):
    """不等于"""
    return float_logic(ctx, l_attr, r_attr, '!=', true_attr, false_attr)


def float_greater(ctx, l_attr, r_attr, true_attr, false_attr):
    """大于"""
    return float_logic(ctx, l_attr, r_attr, '>', true_attr, false_attr)


def float_greater_or_equal(ctx, l_attr, r_attr, true_attr, false_attr):
    """大于等于"""
    return float_logic(ctx, l_attr, r_attr, '>=', true_attr, false_attr)


def float_less(ctx, l_attr, r_attr, true_attr, false_attr):
    """小于"""
    return float_logic(ctx, l_attr, r_attr, '<', true_attr, false_attr)


def float_less_or_equal(ctx, l_attr, r_attr, true_attr, false_attr):
    """小于等于"""
    return float_logic(ctx, l_attr, r_attr, '<=', true_attr, false_attr)


if __name__ == '__main__':
    from rig_core.ctx import Ctx
    from maya_test_tools import question_open_maya_gui

    c = Ctx()

    node = cc.createNode('transform')

    out_attr = float_equal(
        c,
        node.tx, 0,
        1, 0
    )
    print(out_attr, out_attr.get_value())

    question_open_maya_gui()



# -*-coding:utf-8 -*-
"""
:创建时间: 2022/8/4 16:45
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:Github: https://github.com/cpcgskill
:QQ: 2921251087
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127
:爱发电: https://afdian.net/@Phantom_of_the_Cang

3d向量计算图模块
"""
from __future__ import unicode_literals, print_function, division
import cpmel.cmds as cc
from rig_operation_graph._utils import *

__all__ = [
    'add3d', 'sub3d', 'mul3d', 'div3d', 'pow3d', 'mean3d', 'sum3d',
    'vector_add', 'vector_sub', 'vector_mul', 'vector_div', 'vector_pow', 'vector_mean', 'vector_sum'
]


def vector_add(ctx, a, b):
    n = ctx.create_node('plusMinusAverage')
    n.rename('vector_add')
    n['operation'] = 1
    set_or_connect_3d(a, n.input3D[0])
    set_or_connect_3d(b, n.input3D[1])
    return n.output3D


add3d = vector_add


def vector_sub(ctx, a, b):
    n = ctx.create_node('plusMinusAverage')
    n.rename('vector_sub')
    n['operation'] = 2
    set_or_connect_3d(a, n.input3D[0])
    set_or_connect_3d(b, n.input3D[1])
    return n.output3D


sub3d = vector_sub


def vector_mul(ctx, a, b):
    n = ctx.create_node('multiplyDivide')
    n.rename('vector_mul')
    n['operation'] = 1
    set_or_connect_3d(a, n.input1)
    set_or_connect_3d(b, n.input2)
    return n.output


mul3d = vector_mul


def vector_div(ctx, a, b):
    n = ctx.create_node('multiplyDivide')
    n.rename('vector_div')
    n['operation'] = 2
    set_or_connect_3d(a, n.input1)
    set_or_connect_3d(b, n.input2)
    return n.output


div3d = vector_div


def vector_pow(ctx, a, b):
    n = ctx.create_node('multiplyDivide')
    n.rename('vector_pow')
    n['operation'] = 3
    set_or_connect_3d(a, n.input1)
    set_or_connect_3d(b, n.input2)
    return n.output


pow3d = vector_pow


def vector_mean(ctx, *values):
    n = ctx.create_node('plusMinusAverage')
    n.rename('vector_mean')
    n['operation'] = 3
    for id_, v in enumerate(values):
        set_or_connect_3d(v, n.input3D[id_])
    return n.output3D


mean3d = vector_mean


def vector_sum(ctx, *values):
    n = ctx.create_node('plusMinusAverage')
    n.rename('vector_sum')
    n['operation'] = 1
    for id_, v in enumerate(values):
        set_or_connect_3d(v, n.input3D[id_])
    return n.output3D


sum3d = vector_sum

if __name__ == "__main__":
    from rig_core.ctx import Ctx
    from maya_test_tools import question_open_maya_gui

    c = Ctx()

    out_attr = vector_add(c, (1, 1, 1), (-1, -1, -1))
    print(out_attr, out_attr.get_value())

    out_attr = vector_sub(c, out_attr, (-1, 1, -1))
    print(out_attr, out_attr.get_value())

    out_attr = vector_mul(c, out_attr, (1.5, 1.5, 1.5))
    print(out_attr, out_attr.get_value())

    out_attr = vector_div(c, out_attr, (3, 3, 3))
    print(out_attr, out_attr.get_value())

    out_attr = vector_pow(c, out_attr, (3, 3, 3))
    print(out_attr, out_attr.get_value())

    out_attr = vector_mean(c, out_attr, (0, 0, 0), (0, 0, 0), (0, 0, 0))
    print(out_attr, out_attr.get_value())

    out_attr = vector_sum(c, out_attr, out_attr, out_attr, out_attr)
    print(out_attr, out_attr.get_value())

    # question_open_maya_gui()



# -*-coding:utf-8 -*-
"""
:创建时间: 2022/8/4 22:45
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:Github: https://github.com/cpcgskill
:QQ: 2921251087
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127
:爱发电: https://afdian.net/@Phantom_of_the_Cang

"""
from __future__ import unicode_literals, print_function, division
from rig_operation_graph._utils import *

__all__ = ['distance']


def distance(ctx, a, b):
    node = ctx.create_node('distanceBetween')
    node.rename('distance')
    set_or_connect_3d(a, node.point1)
    set_or_connect_3d(b, node.point2)
    return node.distance


if __name__ == '__main__':
    from rig_core.ctx import Ctx

    c = Ctx()
    out_attr = distance(c, (-1, -1, -1), (1, 1, 1))
    print(out_attr, out_attr.get_value())



# -*-coding:utf-8 -*-
u"""
:创建时间: 2022/5/29 6:50
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:QQ: 2921251087
:爱发电: https://afdian.net/@Phantom_of_the_Cang
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127

"""
from __future__ import unicode_literals, print_function, division

import sys

import cpmel.cmds as cc

unicode_t = type('')
bytes_t = type(b'')

if sys.version_info.major < 3:
    num_t = (int, long, float)
else:
    num_t = (int, float)


def set_or_connect(in_value, out_attr):
    if isinstance(in_value, (unicode_t, bytes_t, cc.Attr)):
        in_attr = cc.new_object(in_value)
        in_attr >> out_attr
        return
    out_attr.set_value(in_value)


def set_or_connect_3d(in_value, out_attr):
    if isinstance(in_value, (unicode_t, bytes_t, cc.Attr)):
        in_attr = cc.new_object(in_value)
        in_attr >> out_attr
        return

    attr_plug = out_attr.api1_m_plug()
    out_attr_list = [cc.new_object(attr_plug.child(i).name()) for i in range(attr_plug.numChildren())]

    set_or_connect(in_value[0], out_attr_list[0])
    set_or_connect(in_value[1], out_attr_list[1])
    set_or_connect(in_value[2], out_attr_list[2])


def set_or_connect_matrix(in_value, out_attr):
    if isinstance(in_value, (unicode_t, bytes_t, cc.Attr)):
        in_attr = cc.new_object(in_value)
        in_attr >> out_attr
        return
    out_attr.set_value(in_value)


__all__ = ['set_or_connect', 'set_or_connect_3d', 'set_or_connect_matrix']



# -*-coding:utf-8 -*-
u"""
:创建时间: 2022/5/11 23:07
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:QQ: 2921251087
:爱发电: https://afdian.net/@Phantom_of_the_Cang
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127

反转模块
"""
from __future__ import unicode_literals, print_function, division

import sys

import cpmel.cmds as cc

from rig_operation_graph._utils import *


def reverse(ctx, val):
    node = cc.createNode('reverse')
    set_or_connect(val, node.inputX)
    return node.outputX



# -*-coding:utf-8 -*-
u"""
:创建时间: 2022/5/11 21:38
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:QQ: 2921251087
:爱发电: https://afdian.net/@Phantom_of_the_Cang
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127

驱动模块
"""
from __future__ import unicode_literals, print_function, division

import sys

import cpmel.cmds as cc

from rig_operation_graph._utils import *


def range(ctx, drive, old_min=0.0, old_max=1.0, new_min=0.0, new_max=1.0):
    node = cc.createNode('setRange')
    set_or_connect(old_min, node.oldMinX)
    set_or_connect(old_max, node.oldMaxX)
    set_or_connect(new_min, node.minX)
    set_or_connect(new_max, node.maxX)
    drive >> node.valueX
    return node.outValueX



# -*-coding:utf-8 -*-
u"""
:创建时间: 2022/4/1 20:03
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:QQ: 2921251087
:爱发电: https://afdian.net/@Phantom_of_the_Cang
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127

矩阵模块
"""
from __future__ import unicode_literals, print_function, division
import cpmel.cmds as cc
from rig_operation_graph._utils import *

is_load, _, _ = cc.pluginInfo('matrixNodes', q=True, set=True)
if not is_load:
    print("loading matrixNodes.mll")
    cc.loadPlugin('matrixNodes', qt=True)

__all__ = [
    'new_matrix_from_transform', 'new_matrix_from_transform_attr', 'new_matrix_from_transform_value',
    'copy_matrix',
    'add_matrix', 'mult_matrix',
    'inverse_matrix', 'pass_matrix',
    'matrix_to_transform_attr', 'to_transform_attr',
]


def new_matrix_from_transform(ctx, t=None, r=None, s=None, sh=None):
    n = ctx.create_node('composeMatrix')
    n.rename('new_matrix_from_transform')
    if t is not None:
        set_or_connect_3d(t, n.attr('inputTranslate'))
    if r is not None:
        set_or_connect_3d(r, n.attr('inputRotate'))
    if s is not None:
        set_or_connect_3d(s, n.attr('inputScale'))
    if sh is not None:
        set_or_connect_3d(sh, n.attr('inputShear'))
    return n.attr('outputMatrix')


new_matrix_from_transform_attr = new_matrix_from_transform


def new_matrix_from_transform_value(ctx, t, r, s, sh=(0.0, 0.0, 0.0)):
    n = ctx.create_node('composeMatrix')
    n.rename('new_matrix_from_transform')
    n['inputTranslate'] = t
    n['inputRotate'] = r
    n['inputScale'] = s
    n['inputShear'] = sh
    # n['inputRotateOrder'] = ro
    return n.attr('outputMatrix')


def copy_matrix(ctx, m):
    n = ctx.create_node('decomposeMatrix')
    n.rename('copy_matrix')
    n['inputMatrix'] = m.get_value()
    t, r, s, sh = (
        n.attr('outputTranslate').get_value(),
        n.attr('outputRotate').get_value(),
        n.attr('outputScale').get_value(),
        n.attr('outputShear').get_value()
    )
    cc.delete(n)
    return new_matrix_from_transform_value(ctx, t, r, s, sh)


def add_matrix(ctx, *m_list):
    n = ctx.create_node('addMatrix')
    n.rename('add_matrix')
    for i, a in enumerate(m_list):
        set_or_connect_matrix(a, n.attr('matrixIn[{}]'.format(i)))
    return n.attr('matrixSum')


def mult_matrix(ctx, *m_list):
    n = ctx.create_node('multMatrix')
    n.rename('mult_matrix')
    for i, a in enumerate(m_list):
        set_or_connect_matrix(a, n.attr('matrixIn[{}]'.format(i)))
    return n.attr('matrixSum')


def inverse_matrix(ctx, m):
    n = ctx.create_node('inverseMatrix')
    n.rename('inverse_matrix')
    set_or_connect_matrix(m, n.attr('inputMatrix'))
    return n.attr('outputMatrix')


def pass_matrix(ctx, m, s):
    n = ctx.create_node('passMatrix')
    n.rename('pass_matrix')
    set_or_connect_matrix(m, n.inMatrix)
    set_or_connect(s, n.inScale)
    return n.outMatrix


def matrix_to_transform_attr(ctx, a):
    n = ctx.create_node('decomposeMatrix')
    n.rename('matrix_to_transform')
    set_or_connect_matrix(a, n.attr('inputMatrix'))
    return n.attr('outputTranslate'), n.attr('outputRotate'), n.attr('outputScale'), n.attr('outputShear')


to_transform_attr = matrix_to_transform_attr

if __name__ == "__main__":
    from rig_core.ctx import Ctx

    cc.eval("""
file -f -new;
""")
    ctx = Ctx()
    a = new_matrix_from_transform_value(ctx, (1, 5, 6), (360, 0, 0), (1, 1, 1), (0, 0, 0))
    b = new_matrix_from_transform_value(ctx, (1, 5, 6), (360, 0, 0), (1, 1, 1), (0, 0, 0))

    out_attr = add_matrix(ctx, a, b)
    print("add_matrix", out_attr, 'value == ', out_attr.get_value())

    out_attr = mult_matrix(ctx, a, b)
    print("mult_matrix", out_attr, 'value == ', out_attr.get_value())

    out_attr = inverse_matrix(ctx, a)
    print("inverse_matrix", out_attr, 'value == ', out_attr.get_value())

    out_attr = pass_matrix(ctx, a, 2)
    print("pass_matrix", out_attr, 'value == ', out_attr.get_value())

    out_attr = copy_matrix(ctx, a)
    print("copy_matrix", out_attr, 'value == ', out_attr.get_value())
    print('check org matrix == new matrix', a.get_value() == out_attr.get_value())

    out_attr = to_transform_attr(ctx, a)
    print("to_transform_attr", out_attr)



# -*-coding:utf-8 -*-
"""
:创建时间: 2022/8/23 8:54
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:Github: https://github.com/cpcgskill
:QQ: 2921251087
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127
:爱发电: https://afdian.net/@Phantom_of_the_Cang

"""
from __future__ import unicode_literals, print_function, division
import cpmel.cmds as cc
from rig_operation_graph._utils import *

__all__ = ['float_gauss', 'float_noise', 'float_rand']


def float_gauss(ctx, value):
    exp = cc.expression(s='', o='', ae=True, uc='all')
    ctx.add_nodes(exp)
    exp.rename('float_gauss')
    input_attr = ctx.add_attribute(exp, 'input_value')
    output_attr = ctx.add_attribute(exp, 'output_value')
    cc.expression(
        exp,
        e=True,
        s="{} = gauss({})".format(output_attr.name(), input_attr.name())
    )
    set_or_connect(value, input_attr)
    return output_attr


def float_noise(ctx, value):
    exp = cc.expression(s='', o='', ae=True, uc='all')
    ctx.add_nodes(exp)
    exp.rename('float_noise')
    input_attr = ctx.add_attribute(exp, 'input_value')
    output_attr = ctx.add_attribute(exp, 'output_value')
    cc.expression(
        exp,
        e=True,
        s="{} = noise({})".format(output_attr.name(), input_attr.name())
    )
    set_or_connect(value, input_attr)
    return output_attr


def float_rand(ctx, min, max):
    exp = cc.expression(s='', o='', ae=True, uc='all')
    ctx.add_nodes(exp)
    exp.rename('float_rand')
    input_min_attr = ctx.add_attribute(exp, 'input_min_value')
    input_max_attr = ctx.add_attribute(exp, 'input_max_value')
    output_attr = ctx.add_attribute(exp, 'output_value')
    cc.expression(
        exp,
        e=True,
        s="{} = rand({}, {})".format(output_attr.name(), input_min_attr.name(), input_max_attr.name())
    )
    set_or_connect(min, input_min_attr)
    set_or_connect(max, input_max_attr)
    return output_attr



# -*-coding:utf-8 -*-
u"""
:创建时间: 2022/5/11 22:41
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:QQ: 2921251087
:爱发电: https://afdian.net/@Phantom_of_the_Cang
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127

限制模块
"""
from __future__ import unicode_literals, print_function, division

import sys

import cpmel.cmds as cc

from rig_operation_graph._utils import *

from rig_core.all import *

if False:
    from typing import AnyStr, Any, List, Tuple, Dict

__all__ = ['clamp']


def clamp(ctx, input, min=0.0, max=1.0):
    """
    限制最大最小值

    :type ctx: Ctx
    :type input: cc.Attr or float
    :type min: cc.Attr or float
    :type max: cc.Attr or float
    :rtype: cc.Attr
    """
    node = cc.createNode('clamp')
    set_or_connect(min, node.minR)
    set_or_connect(max, node.maxR)
    set_or_connect(input, node.inputR)
    return node.outputR


if __name__ == '__main__':
    def test():
        from maya_test_tools import open_file, question_open_maya_gui

        ctx = Ctx()

        print('clamp', clamp(ctx, 1.5, 0, 0.8).value)

        question_open_maya_gui()


    test()



from cpmel.cmds import Attr
from rig_core.all import *
from rig_operation_graph._utils import *

from typing import Tuple, Union, Any

Number = Union[float, int]
Vector3d = Tuple[Union[float, int], Union[float, int], Union[float, int]]
InValue = Union[Vector3d, Attr, Any]
OutAttr = Attr

__all__ = [
    'add3d', 'sub3d', 'mul3d', 'div3d', 'pow3d', 'mean3d', 'sum3d',
    'vector_add', 'vector_sub', 'vector_mul', 'vector_div', 'vector_pow', 'vector_mean', 'vector_sum'
]


def add3d(ctx: Ctx, a: InValue, b: InValue) -> OutAttr: pass


def sub3d(ctx: Ctx, a: InValue, b: InValue) -> OutAttr: pass


def mul3d(ctx: Ctx, a: InValue, b: InValue) -> OutAttr: pass


def div3d(ctx: Ctx, a: InValue, b: InValue) -> OutAttr: pass


def pow3d(ctx: Ctx, a: InValue, b: InValue) -> OutAttr: pass


def mean3d(ctx: Ctx, *values: InValue) -> OutAttr: pass


def sum3d(ctx: Ctx, *values: InValue) -> OutAttr: pass


def vector_add(ctx: Ctx, a: InValue, b: InValue) -> OutAttr: pass


def vector_sub(ctx: Ctx, a: InValue, b: InValue) -> OutAttr: pass


def vector_mul(ctx: Ctx, a: InValue, b: InValue) -> OutAttr: pass


def vector_div(ctx: Ctx, a: InValue, b: InValue) -> OutAttr: pass


def vector_pow(ctx: Ctx, a: InValue, b: InValue) -> OutAttr: pass


def vector_mean(ctx: Ctx, *values: InValue) -> OutAttr: pass


def vector_sum(ctx: Ctx, *values: InValue) -> OutAttr: pass
